@@ -1,14 +1,14 @@
 #' Convert KFAS Model to bssm Model
 #'
-#' Converts \code{SSModel} object of \code{KFAS} package to 
-#' \code{bssm} model.
+#' Converts \code{SSModel} object of \code{KFAS} package to general
+#' \code{bssm} model of type \code{ssm_ulg}, \code{ssm_mlg}, \code{ssm_ung} or \code{ssm_mng}.
 #' 
 #' @param model Object of class \code{SSModel}.
 #' @param kappa For \code{SSModel} object, a prior variance for initial state
 #' used to replace exact diffuse elements of the original model.
-#' @param ... Additional arguments to \code{ssm_mlg} and \code{ssm_mng} 
+#' @param ... Additional arguments to model building functions of \code{bssm}
 #' (such as prior and updating functions).
-#' @return Object of class \code{ssm_mlg} or \code{ssm_mng}.
+#' @return Object of class \code{ssm_ulg}, \code{ssm_mlg}, \code{ssm_ung} or \code{ssm_mng}.
 #' @export
 as_bssm <- function(model, kappa = 100, ...) {
   
@@ -91,13 +91,13 @@ as_bssm <- function(model, kappa = 100, ...) {
           },
           gamma = {
             if(length(unique(model$u[,i])) > 1)
-              stop("Time-varying shape parameter for gamma is not supported in 'bssm'.")
+              stop("Time-varying shape parameter for gamma is not (yet) supported in 'bssm'.")
             phi[i] <- model$u[1,i]
             u[,i] <- 1
           },
           "negative binomial" = {
             if(length(unique(model$u[,i])) > 1)
-              stop("Time-varying dispersion parameter for negative binomial is not supported in 'bssm'.")
+              stop("Time-varying dispersion parameter for negative binomial is not (yet) supported in 'bssm'.")
             phi[i] <- model$u[1,i]
             u[,i] <- 1
           }, 
