@@ -3,7 +3,7 @@
 #' Function \code{ekpf_filter} performs a extended Kalman particle filtering with stratification
 #' resampling, based on Van Der Merwe et al (2001).
 #'
-#' @param object of class \code{nlg_ssm}.
+#' @param object of class \code{ssm_nlg}.
 #' @param nsim Number of samples.
 #' @param seed Seed for RNG.
 #' @param ... Ignored.
@@ -15,17 +15,17 @@
 ekpf_filter <- function(object, nsim, ...) {
   UseMethod("ekpf_filter", object)
 }
-#' @method ekpf_filter nlg_ssm
+#' @method ekpf_filter ssm_nlg
 #' @export
 #' @rdname ekpf_filter
-ekpf_filter.nlg_ssm <- function(object, nsim, seed = sample(.Machine$integer.max, size = 1), ...) {
+ekpf_filter.ssm_nlg <- function(object, nsim, seed = sample(.Machine$integer.max, size = 1), ...) {
   
   out <- ekpf(t(object$y), object$Z, object$H, object$T, 
     object$R, object$Z_gn, object$T_gn, object$a1, object$P1, 
     object$theta, object$log_prior_pdf, object$known_params, 
     object$known_tv_params, object$n_states, object$n_etas, 
     as.integer(object$time_varying), nsim, 
-    seed)
+    seed, default_update_fn, default_prior_fn)
   colnames(out$at) <- colnames(out$att) <- colnames(out$Pt) <-
     colnames(out$Ptt) <- rownames(out$Pt) <- rownames(out$Ptt) <- 
     rownames(out$alpha) <- object$state_names
@@ -33,4 +33,4 @@ ekpf_filter.nlg_ssm <- function(object, nsim, seed = sample(.Machine$integer.max
   out$att <- ts(out$att, start = start(object$y), frequency = frequency(object$y))
   out$alpha <- aperm(out$alpha, c(2, 1, 3))
   out
-}
\ No newline at end of file
+}
