@@ -7,6 +7,7 @@
 #include "model_svm.h"
 #include "model_ar1_ng.h"
 #include "model_ssm_nlg.h"
+#include "model_ssm_mng.h"
 
 #include "filter_smoother.h"
 #include "summary.h"
@@ -14,49 +15,52 @@
 Rcpp::List bsf(const Rcpp::List model_,
   const unsigned int nsim, const unsigned int seed,
   bool gaussian, const int model_type) {
-
+  
   if (gaussian) {
     switch (model_type) {
-    case 1: {
-  ssm_ulg model(model_, seed);
-  unsigned int m = model.m;
-  unsigned n = model.n;
-
-  arma::cube alpha(m, n + 1, nsim);
-  arma::mat weights(nsim, n + 1);
-  arma::umat indices(nsim, n);
-  double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
-  arma::mat at(m, n + 1);
-  arma::mat att(m, n);
-  arma::cube Pt(m, m, n + 1);
-  arma::cube Ptt(m, m, n);
-  filter_summary(alpha, at, att, Pt, Ptt, weights);
-
-  arma::inplace_trans(at);
-  arma::inplace_trans(att);
-  return Rcpp::List::create(
-    Rcpp::Named("at") = at, Rcpp::Named("att") = att,
-    Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
-    Rcpp::Named("weights") = weights,
-    Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+    case 0: {
+  Rcpp::stop("Bootstrap particle filtering for multivariate gaussian model is not yet supported. ");
 } break;
-    case 2: {
-      bsm_lg model(model_, seed);
+    case 1: {
+      ssm_ulg model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
+      
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
       arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+      
+      arma::inplace_trans(at);
+      arma::inplace_trans(att);
+      return Rcpp::List::create(
+        Rcpp::Named("at") = at, Rcpp::Named("att") = att,
+        Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+    } break;
+    case 2: {
+      bsm_lg model(model_, seed);
+      unsigned int m = model.m;
+      unsigned n = model.n;
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      
+      arma::mat at(m, n + 1);
+      arma::mat att(m, n);
+      arma::cube Pt(m, m, n + 1);
+      arma::cube Ptt(m, m, n);
+      filter_summary(alpha, at, att, Pt, Ptt, weights);
+      
       arma::inplace_trans(at);
       arma::inplace_trans(att);
       return Rcpp::List::create(
@@ -69,18 +73,18 @@ Rcpp::List bsf(const Rcpp::List model_,
       ar1_lg model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
+      
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
       arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+      
       arma::inplace_trans(at);
       arma::inplace_trans(att);
       return Rcpp::List::create(
@@ -92,22 +96,22 @@ Rcpp::List bsf(const Rcpp::List model_,
     }
   } else {
     switch (model_type) {
-    case 1: {
-    ssm_ung model(model_, seed);
+    case 0: {
+    ssm_mng model(model_, seed);
     unsigned int m = model.m;
     unsigned n = model.n;
-
+    
     arma::cube alpha(m, n + 1, nsim);
     arma::mat weights(nsim, n + 1);
     arma::umat indices(nsim, n);
     double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
+    
     arma::mat at(m, n + 1);
     arma::mat att(m, n);
     arma::cube Pt(m, m, n + 1);
     arma::cube Ptt(m, m, n);
     filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+    
     arma::inplace_trans(at);
     arma::inplace_trans(att);
     return Rcpp::List::create(
@@ -116,11 +120,35 @@ Rcpp::List bsf(const Rcpp::List model_,
       Rcpp::Named("weights") = weights,
       Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
   } break;
+    case 1: {
+      ssm_ung model(model_, seed);
+      unsigned int m = model.m;
+      unsigned n = model.n;
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      
+      arma::mat at(m, n + 1);
+      arma::mat att(m, n);
+      arma::cube Pt(m, m, n + 1);
+      arma::cube Ptt(m, m, n);
+      filter_summary(alpha, at, att, Pt, Ptt, weights);
+      
+      arma::inplace_trans(at);
+      arma::inplace_trans(att);
+      return Rcpp::List::create(
+        Rcpp::Named("at") = at, Rcpp::Named("att") = att,
+        Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+    } break;
     case 2: {
       bsm_ng model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
@@ -130,7 +158,7 @@ Rcpp::List bsf(const Rcpp::List model_,
       arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+      
       arma::inplace_trans(at);
       arma::inplace_trans(att);
       return Rcpp::List::create(
@@ -138,13 +166,13 @@ Rcpp::List bsf(const Rcpp::List model_,
         Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+      
     } break;
     case 3: {
       svm model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
@@ -154,7 +182,7 @@ Rcpp::List bsf(const Rcpp::List model_,
       arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+      
       arma::inplace_trans(at);
       arma::inplace_trans(att);
       return Rcpp::List::create(
@@ -162,13 +190,13 @@ Rcpp::List bsf(const Rcpp::List model_,
         Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+      
     } break;
     case 4: {
       ar1_ng model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
@@ -178,7 +206,7 @@ Rcpp::List bsf(const Rcpp::List model_,
       arma::cube Pt(m, m, n + 1);
       arma::cube Ptt(m, m, n);
       filter_summary(alpha, at, att, Pt, Ptt, weights);
-
+      
       arma::inplace_trans(at);
       arma::inplace_trans(att);
       return Rcpp::List::create(
@@ -186,7 +214,7 @@ Rcpp::List bsf(const Rcpp::List model_,
         Rcpp::Named("Pt") = Pt, Rcpp::Named("Ptt") = Ptt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+      
     } break;
     }
   }
@@ -198,165 +226,190 @@ Rcpp::List bsf(const Rcpp::List model_,
 Rcpp::List bsf_smoother(const Rcpp::List model_,
   const unsigned int nsim, const unsigned int seed,
   bool gaussian, const int model_type) {
-
+  
   if (gaussian) {
-    switch (model_type) {
+    switch (model_type) {  
+    case 0: {
+  Rcpp::stop("Bootstrap particle smoothing for multivariate gaussian model is not yet supported. ");
+} break;
     case 1: {
       ssm_ulg model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
+      
+      filter_smoother(alpha, indices);
+      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      
+      arma::inplace_trans(alphahat);
+      return Rcpp::List::create(
+        Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+    } break;
+    case 2: {
+      bsm_lg model(model_, seed);
+      unsigned int m = model.m;
+      unsigned n = model.n;
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
+      
+      filter_smoother(alpha, indices);
+      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      
+      arma::inplace_trans(alphahat);
+      return Rcpp::List::create(
+        Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+      
+    } break;
+    case 3: {
+      ar1_lg model(model_, seed);
+      unsigned int m = model.m;
+      unsigned n = model.n;
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
+      
+      filter_smoother(alpha, indices);
+      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      
+      arma::inplace_trans(alphahat);
+      return Rcpp::List::create(
+        Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+      
+    } break;
+    }
+  } else {
+    switch (model_type) {
+    case 0: {
+    ssm_mng model(model_, seed);
+    unsigned int m = model.m;
+    unsigned n = model.n;
+    
     arma::cube alpha(m, n + 1, nsim);
     arma::mat weights(nsim, n + 1);
     arma::umat indices(nsim, n);
     double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
+    
     arma::mat alphahat(model.m, model.n + 1);
     arma::cube Vt(model.m, model.m, model.n + 1);
-
+    
     filter_smoother(alpha, indices);
     weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
+    
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
       Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
       Rcpp::Named("weights") = weights,
       Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
   } break;
-      case 2: {
-        bsm_lg model(model_, seed);
-        unsigned int m = model.m;
-        unsigned n = model.n;
-
-        arma::cube alpha(m, n + 1, nsim);
-        arma::mat weights(nsim, n + 1);
-        arma::umat indices(nsim, n);
-        double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-        arma::mat alphahat(model.m, model.n + 1);
-        arma::cube Vt(model.m, model.m, model.n + 1);
-
-        filter_smoother(alpha, indices);
-        weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
-        arma::inplace_trans(alphahat);
-        return Rcpp::List::create(
-          Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
-          Rcpp::Named("weights") = weights,
-          Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
-      } break;
-    case 3: {
-        ar1_lg model(model_, seed);
-        unsigned int m = model.m;
-        unsigned n = model.n;
-
-        arma::cube alpha(m, n + 1, nsim);
-        arma::mat weights(nsim, n + 1);
-        arma::umat indices(nsim, n);
-        double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-        arma::mat alphahat(model.m, model.n + 1);
-        arma::cube Vt(model.m, model.m, model.n + 1);
-
-        filter_smoother(alpha, indices);
-        weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
-        arma::inplace_trans(alphahat);
-        return Rcpp::List::create(
-          Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
-          Rcpp::Named("weights") = weights,
-          Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
-      } break;
-      }
-  } else {
-      switch (model_type) {
-      case 1: {
+    case 1: {
       ssm_ung model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
-
+      
       filter_smoother(alpha, indices);
       weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
+      
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
         Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
     } break;
-      case 2: {
-        bsm_ng model(model_, seed);
-        unsigned int m = model.m;
-        unsigned n = model.n;
-
-        arma::cube alpha(m, n + 1, nsim);
-        arma::mat weights(nsim, n + 1);
-        arma::umat indices(nsim, n);
-        double loglik = model.bsf_filter(nsim, alpha, weights, indices);
-        arma::mat alphahat(model.m, model.n + 1);
-        arma::cube Vt(model.m, model.m, model.n + 1);
-
-        filter_smoother(alpha, indices);
-        weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
-        arma::inplace_trans(alphahat);
-        return Rcpp::List::create(
-          Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
-          Rcpp::Named("weights") = weights,
-          Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+    case 2: {
+      bsm_ng model(model_, seed);
+      unsigned int m = model.m;
+      unsigned n = model.n;
+      
+      arma::cube alpha(m, n + 1, nsim);
+      arma::mat weights(nsim, n + 1);
+      arma::umat indices(nsim, n);
+      double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      arma::mat alphahat(model.m, model.n + 1);
+      arma::cube Vt(model.m, model.m, model.n + 1);
+      
+      filter_smoother(alpha, indices);
+      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      
+      arma::inplace_trans(alphahat);
+      return Rcpp::List::create(
+        Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
+        Rcpp::Named("weights") = weights,
+        Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
+      
     } break;
     case 3: {
       svm model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
-
+      
       filter_smoother(alpha, indices);
       weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
+      
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
         Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+      
     } break;
-      case 4: {
+    case 4: {
       ar1_ng model(model_, seed);
       unsigned int m = model.m;
       unsigned n = model.n;
-
+      
       arma::cube alpha(m, n + 1, nsim);
       arma::mat weights(nsim, n + 1);
       arma::umat indices(nsim, n);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
-
+      
       filter_smoother(alpha, indices);
       weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
-
+      
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
         Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
         Rcpp::Named("weights") = weights,
         Rcpp::Named("logLik") = loglik, Rcpp::Named("alpha") = alpha);
-
+      
     } break;
     }
   }
@@ -372,8 +425,8 @@ Rcpp::List bsf_nlg(const arma::mat& y, SEXP Z, SEXP H,
   const unsigned int n_etas,  const arma::uvec& time_varying,
   const unsigned int nsim, const unsigned int seed,
   const Rcpp::Function update_fn, const Rcpp::Function prior_fn) {
-
-
+  
+  
   Rcpp::XPtr<nvec_fnPtr> xpfun_Z(Z);
   Rcpp::XPtr<nmat_fnPtr> xpfun_H(H);
   Rcpp::XPtr<nvec_fnPtr> xpfun_T(T);
@@ -390,7 +443,7 @@ Rcpp::List bsf_nlg(const arma::mat& y, SEXP Z, SEXP H,
   
   unsigned int m = model.m;
   unsigned n = model.n;
-
+  
   arma::cube alpha(m, n + 1, nsim);
   arma::mat weights(nsim, n + 1);
   arma::umat indices(nsim, n);
@@ -419,8 +472,8 @@ Rcpp::List bsf_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   const unsigned int n_etas,  const arma::uvec& time_varying,
   const unsigned int nsim, const unsigned int seed,
   const Rcpp::Function update_fn, const Rcpp::Function prior_fn) {
-
-
+  
+  
   Rcpp::XPtr<nvec_fnPtr> xpfun_Z(Z);
   Rcpp::XPtr<nmat_fnPtr> xpfun_H(H);
   Rcpp::XPtr<nvec_fnPtr> xpfun_T(T);
@@ -434,10 +487,10 @@ Rcpp::List bsf_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   ssm_nlg model(y, *xpfun_Z, *xpfun_H, *xpfun_T, *xpfun_R, *xpfun_Zg, *xpfun_Tg,
     *xpfun_a1, *xpfun_P1,  theta, *xpfun_prior, known_params, known_tv_params, n_states, n_etas,
     time_varying, update_fn, prior_fn, seed);
-
+  
   unsigned int m = model.m;
   unsigned n = model.n;
-
+  
   arma::cube alpha(m, n + 1, nsim);
   arma::mat weights(nsim, n + 1);
   arma::umat indices(nsim, n);
@@ -448,7 +501,7 @@ Rcpp::List bsf_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   filter_smoother(alpha, indices);
   weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
   arma::inplace_trans(alphahat);
-
+  
   return Rcpp::List::create(
     Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
     Rcpp::Named("weights") = weights,
